
class ProgressConstants:
    """进度相关常量"""
    MIN_PROGRESS = 0.0
    MAX_PROGRESS = 100.0
    BATCH_UPDATE_SIZE = 100

    # 上传步骤的内部进度分配
    UPLOAD_SAVE_DETAIL = 30
    UPLOAD_S3_DETAIL = 70
    UPLOAD_S3_CHARTS = 90





from enum import Enum

class TaskStatus(Enum):
    """任务状态枚举"""
    PENDING = "pending"
    RUNNING = "running"
    COMPLETED = "completed"
    FAILED = "failed"
    CANCELLED = "cancelled"

    @property
    def display_name(self) -> str:
        """获取显示名称"""
        display_names = {
            TaskStatus.PENDING: "等待中",
            TaskStatus.RUNNING: "运行中",
            TaskStatus.COMPLETED: "已完成",
            TaskStatus.FAILED: "失败",
            TaskStatus.CANCELLED: "已取消"
        }
        return display_names.get(self, self.value)

    def __str__(self) -> str:
        return self.value

    @classmethod
    def from_value(cls, value: str) -> 'TaskStatus':
        """从字符串值获取枚举"""
        for status in cls:
            if status.value == value:
                return status
        raise ValueError(f"无效的任务状态: {value}")


class EvaluationStep(Enum):
    """评估步骤枚举"""
    DATA_LOADING = ("数据加载", 0, 10)
    EVALUATION = ("执行评估", 10, 90)
    CHART_GENERATION = ("图表生成", 90, 95)
    RESULT_UPLOAD = ("结果上传", 95, 100)

    def __init__(self, display_name: str, start_progress: int, end_progress: int):
        self.display_name = display_name
        self.start_progress = start_progress
        self.end_progress = end_progress

    def calculate_overall_progress(self, step_progress: float) -> int:
        """将步骤内进度转换为总体进度"""
        step_progress = max(0.0, min(100.0, step_progress))
        progress_range = self.end_progress - self.start_progress
        return int(self.start_progress + progress_range * step_progress / 100.0)



import os
import threading
import json
import traceback
import pandas as pd
from datetime import datetime
from abc import ABC, abstractmethod
from typing import Dict, Any, List, Callable, Optional

from libentry.mcp import api
from libentry import logger

from evaluation_service_base.charts.dtypes import BaseChart
from evaluation_service_base.core.common import BaseEvaluationServiceConfig, EvaluationContext, \
    EvaluationServiceInputParams, EvaluationData, EvaluationResults, BaseEvaluationRaw
from evaluation_service_base.core.constants import ProgressConstants
from evaluation_service_base.core.enum import EvaluationStep, TaskStatus
from evaluation_service_base.core.exceptions import TaskCancelledException
from evaluation_service_base.core.managers import StepProgressManager
from evaluation_service_base.core.reporters import StepProgressReporter
from evaluation_service_base.utils.minio_client import MinioClient, MinioConfig
from evaluation_service_base.utils.s3_handler import S3DataHandler


class BaseEvaluationService(ABC, S3DataHandler):
    """评估服务基类"""

    def __init__(self, config: BaseEvaluationServiceConfig, minio_config: MinioConfig):
        self.config = config
        self.minio_client = MinioClient(minio_config)
        super().__init__(self.minio_client)
        self.progress_manager = StepProgressManager(config)
        self._ensure_directories()

    def _ensure_directories(self) -> None:
        """确保必要的目录存在"""
        directories = [self.config.result_dir, self.config.progress_log_dir]
        for directory in directories:
            os.makedirs(directory, exist_ok=True)

    def _setup_evaluation_context(self, task_id: str, input_params: EvaluationServiceInputParams) -> EvaluationContext:
        """设置评估上下文"""
        context = EvaluationContext(task_id=task_id, input_params=input_params)
        self._setup_local_detail_path(context)
        return context

    def _setup_local_detail_path(self, context: EvaluationContext):
        """设置本地详细结果路径"""
        os.makedirs(self.config.result_dir, exist_ok=True)

        base_filename = f"{context.task_id}_detail"
        json_path = f"{self.config.result_dir}/{base_filename}.json"
        h5_path = f"{self.config.result_dir}/{base_filename}.h5"

        context.set_local_detail_path(json_path)
        context.set_local_h5_detail_path(h5_path)

        logger.info(f"设置本地详细结果路径: {json_path}")
        return

    @api.post()
    def evaluate(self, input_param: EvaluationServiceInputParams) -> Dict[str, Any]:
        """启动评估任务"""
        task_id = input_param.task_id

        if not self.progress_manager.create_task(task_id):
            return self._create_response("already_running", "任务已在运行", task_id)

        threading.Thread(
            target=self._execute_evaluation,
            args=(task_id, input_param),
            daemon=True,
            name=f"eval-{task_id}"
        ).start()

        return self._create_response("started", "任务已启动", task_id)

    def _create_response(self, status: str, message: str, task_id: str) -> Dict[str, Any]:
        """创建统一的响应格式"""
        return {
            "status": status,
            "message": message,
            "task_id": task_id
        }

    def _execute_evaluation(self, task_id: str, input_param: EvaluationServiceInputParams) -> None:
        """执行评估流水线"""
        context = self._setup_evaluation_context(task_id, input_param)

        try:
            cancellation_checker = self._create_cancellation_checker(task_id)
            result = self._run_evaluation_pipeline(context, cancellation_checker)
            self.progress_manager.complete_task(task_id, result)

        except TaskCancelledException:
            logger.info(f"任务 {task_id} 已被取消")
        except Exception as e:
            self._handle_evaluation_error(task_id, e)
        finally:
            self._cleanup_files(context.temp_files)

    def _create_cancellation_checker(self, task_id: str) -> Callable[[], None]:
        """创建取消检查函数"""

        def check_cancelled():
            state = self.progress_manager.get_task_state(task_id)
            if state and state.cancelled:
                raise TaskCancelledException()

        return check_cancelled

    def _handle_evaluation_error(self, task_id: str, error: Exception) -> None:
        """处理评估错误"""
        error_info = {
            "error_type": type(error).__name__,
            "error_message": str(error),
            "traceback": traceback.format_exc(),
            "timestamp": datetime.now().isoformat()
        }
        self.progress_manager.fail_task(task_id, str(error), error_info)
        logger.error(f"任务 {task_id} 失败: {error}", exc_info=True)

    def _cleanup_files(self, file_paths: List[str]):
        """清理临时文件"""
        for file_path in file_paths:
            try:
                if os.path.exists(file_path):
                    os.remove(file_path)
            except Exception as e:
                logger.warning(f"清理文件失败: {file_path}, 错误: {e}")

    def _run_evaluation_pipeline(self, context: EvaluationContext, check_cancelled: Callable) -> Dict[str, Any]:
        """运行评估管道"""
        # 数据加载
        input_data = self._execute_data_loading(context, check_cancelled)

        # 执行评估
        evaluation_results = self._execute_evaluation_step(context, input_data, check_cancelled)

        # 生成图表
        charts = self._execute_chart_generation(context, evaluation_results, check_cancelled)

        # 上传结果
        return self._execute_result_upload(context, evaluation_results, charts, check_cancelled)

    def _execute_data_loading(self, context: EvaluationContext, check_cancelled: Callable) -> EvaluationData:
        """执行数据加载步骤"""
        reporter = StepProgressReporter(self.progress_manager, context.task_id, EvaluationStep.DATA_LOADING)
        reporter.update(0, "开始数据加载")

        input_data = self.load_input_data(context, reporter)
        check_cancelled()

        self.progress_manager.complete_step(context.task_id, EvaluationStep.DATA_LOADING)
        return input_data

    def _execute_evaluation_step(self, context: EvaluationContext, input_data: EvaluationData,
                                 check_cancelled: Callable) -> EvaluationResults:
        """执行评估步骤"""
        reporter = StepProgressReporter(self.progress_manager, context.task_id, EvaluationStep.EVALUATION)
        reporter.update(0, "开始执行评估")

        results = self.run_evaluation(context, input_data, reporter, check_cancelled)
        self.progress_manager.complete_step(context.task_id, EvaluationStep.EVALUATION)
        return results

    def _execute_chart_generation(self, context: EvaluationContext, results: EvaluationResults,
                                  check_cancelled: Callable) -> List[BaseChart]:
        """执行图表生成步骤"""
        reporter = StepProgressReporter(self.progress_manager, context.task_id, EvaluationStep.CHART_GENERATION)
        reporter.update(0, "开始生成图表")

        charts = self.generate_charts(context, results, reporter)
        check_cancelled()

        self.progress_manager.complete_step(context.task_id, EvaluationStep.CHART_GENERATION)
        return charts

    def _execute_result_upload(self, context: EvaluationContext, results: EvaluationResults,
                               charts: List[BaseChart], check_cancelled: Callable) -> Dict[str, Any]:
        """执行结果上传步骤"""
        reporter = StepProgressReporter(self.progress_manager, context.task_id, EvaluationStep.RESULT_UPLOAD)
        reporter.update(0, "开始上传结果")

        # 保存详细结果
        reporter.update(ProgressConstants.UPLOAD_SAVE_DETAIL, "保存详细结果")
        detail_path = self.save_detail_results(context, results, reporter)
        context.add_temp_file(detail_path)
        check_cancelled()

        # 上传详细结果到S3
        reporter.update(ProgressConstants.UPLOAD_S3_DETAIL, "上传详细结果")
        self._put_detail_to_s3(detail_path, context.input_params.result_detail_path)
        check_cancelled()

        # 上传图表结果到S3
        reporter.update(ProgressConstants.UPLOAD_S3_CHARTS, "上传图表结果")
        chart_data = [chart.dict() for chart in charts]
        self._put_json_to_s3(chart_data, context.input_params.result_metric_path)

        self.progress_manager.complete_step(context.task_id, EvaluationStep.RESULT_UPLOAD)

        return {
            "detail_s3_path": context.input_params.result_detail_path,
            "stat_s3_path": context.input_params.result_metric_path,
            "task_id": context.task_id
        }

    def load_input_data(self, context: EvaluationContext, reporter: StepProgressReporter) -> EvaluationData:
        data_list = []

        for chunk_num, df in enumerate(self.read_hdf5_chunks_from_s3(context.input_params.input_path)):
            chunk_list = [
                BaseEvaluationRaw(
                    data_id=row['data_id'],
                    raw_data=json.loads(json.loads(row['_RawData_'])),
                    inputs=json.loads(json.loads(row['inputs'])).get('inputs'),
                    outputs=json.loads(json.loads(row['outputs']))
                )
                for _, row in df.iterrows()
            ]
            data_list.extend(chunk_list)

        return EvaluationData(data_list=data_list)

    def save_detail_results(self, context: EvaluationContext, results: EvaluationResults,
                            reporter: StepProgressReporter) -> str:
        """保存详细评估结果到H5文件"""
        try:
            reporter.update(0, "开始保存详细结果")

            # 确保路径已设置
            if not context.local_detail_path:
                self._setup_local_detail_path(context)

            # 转换数据
            results_data = self._convert_results_to_records(results.results, reporter)

            # 保存到H5文件
            h5_path = self._save_results_to_h5(results_data, context, reporter)

            reporter.update(100, f"详细结果已保存到: {h5_path}")
            logger.info(f"详细结果保存完成: {h5_path}, 记录数: {len(results.results)}")

            return h5_path

        except Exception as e:
            error_msg = f"保存详细结果失败: {str(e)}"
            logger.error(error_msg, exc_info=True)
            reporter.update(0, error_msg)
            raise

    def _convert_results_to_records(self, results: List[Any], reporter: StepProgressReporter) -> List[Dict]:
        """转换结果为记录列表"""
        records = []
        total_count = len(results)

        for idx, result in enumerate(results):
            record = result.model_dump() if hasattr(result, 'model_dump') else result.dict()
            records.append(record)

            # 批量更新进度
            if idx % ProgressConstants.BATCH_UPDATE_SIZE == 0:
                progress = (idx / total_count) * 70  # 70%用于数据转换
                reporter.update(progress, f"处理数据 {idx}/{total_count}")

        reporter.update(70, "数据转换完成")
        return records

    def _save_results_to_h5(self, records: List[Dict], context: EvaluationContext,
                            reporter: StepProgressReporter) -> str:
        """保存记录到H5文件"""
        reporter.update(75, "创建DataFrame")
        df = pd.DataFrame(records)

        reporter.update(85, "保存到H5文件")
        h5_path = context.local_h5_detail_path
        self.append_to_hdf5(df, h5_path, key='df')

        return h5_path

    @api.post()
    def progress(self, task_id: str) -> Dict[str, Any]:
        """获取任务进度"""
        state = self.progress_manager.get_task_state(task_id)
        if not state:
            return {
                "progress": 0,
                "status": TaskStatus.PENDING.value,  # 🆕 使用枚举值
                "status_display": "任务不存在",
                "message": "任务不存在"
            }

        result = state.to_dict()

        # 根据状态添加额外信息
        if state.status == TaskStatus.COMPLETED and state.result:
            result["result"] = state.result
        elif state.status == TaskStatus.FAILED and state.error_info:
            result["error_detail"] = state.error_info

        return result

    @api.post()
    def cancel(self, task_id: str) -> Dict[str, Any]:
        """取消任务"""
        state = self.progress_manager.get_task_state(task_id)

        if not state:
            return {
                "status": "not_found",
                "message": "任务不存在",
                "task_id": task_id
            }

        if self.progress_manager.cancel_task(task_id):
            return {
                "status": TaskStatus.CANCELLED.value,  # 🆕 使用枚举值
                "message": "任务已成功取消",
                "task_id": task_id
            }

        # 检查当前状态
        current_status = state.status
        if current_status == TaskStatus.CANCELLED:
            return {
                "status": "already_cancelled",
                "message": f"任务已经被取消 (状态: {current_status.display_name})",
                "task_id": task_id
            }
        elif current_status in [TaskStatus.COMPLETED, TaskStatus.FAILED]:
            return {
                "status": "cannot_cancel",
                "message": f"任务已结束，无法取消 (状态: {current_status.display_name})",
                "task_id": task_id
            }
        else:
            return {
                "status": "not_running",
                "message": f"任务未在运行中 (状态: {current_status.display_name})",
                "task_id": task_id
            }

    @api.post()
    def get_task_status(self, task_id: str) -> Dict[str, Any]:
        """🆕 获取任务状态详情（新增接口）"""
        state = self.progress_manager.get_task_state(task_id)
        if not state:
            return {"error": "任务不存在"}

        return {
            "task_id": task_id,
            "status": {
                "value": state.status.value,
                "display_name": state.status.display_name,
                "is_active": state.is_active(),
                "is_finished": state.is_finished(),
                "can_be_cancelled": state.can_be_cancelled()
            },
            "progress": state.overall_progress,
            "message": state.message,
            "created_at": state.created_at.isoformat(),
            "updated_at": state.updated_at.isoformat()
        }

    @abstractmethod
    def run_evaluation(self, context: EvaluationContext, data: EvaluationData,
                       reporter: StepProgressReporter, check_cancelled: Callable) -> EvaluationResults:
        """执行评估 - 子类通过reporter.update(progress, message)更新进度"""
        pass

    @abstractmethod
    def generate_charts(self, context: EvaluationContext, result: Any, reporter: StepProgressReporter) -> List[
        BaseChart]:
        """生成图表 - 子类通过reporter.update(progress, message)更新进度"""
        pass



from evaluation_service_base.core.enum import EvaluationStep
from evaluation_service_base.core.managers import StepProgressManager


class StepProgressReporter:
    """步骤进度报告器 - 传递给子类使用"""

    def __init__(self, manager: StepProgressManager, task_id: str, step: EvaluationStep):
        self.manager = manager
        self.task_id = task_id
        self.step = step

    def update(self, progress: float, message: str = ""):
        """更新当前步骤的进度 - 子类只需要关注这个方法"""
        self.manager.update_step_progress(self.task_id, self.step, progress, message)




#!/usr/bin/env python3
from typing import Optional, List, Dict
from pydantic import BaseModel, ConfigDict, Field


class EvaluationServiceInputParams(BaseModel):
    task_id: str = Field(description="任务ID")
    input_path: Optional[str] = Field(
        description="推理结果S3地址",
        default="tool_evaluation/infer_Result/Dataset7.n_RAG_e2e.json"
    )
    result_detail_path: Optional[str] = Field(
        description="评估结果S3地址", default="tool_evaluation/result_detail/Dataset7.n_RAG_e2e.json"
    )
    result_metric_path: Optional[str] = Field(
        description="指标结果S3地址",
        default="tool_evaluation/result_metrics/Dataset7.n_RAG_e2e.json"
    )
    model_config = ConfigDict(extra="allow")


class BaseEvaluationServiceConfig(BaseModel):
    progress_log_dir: str = Field(default="./data/progress", description="进度日志目录")
    result_dir: str = Field(default="./data/results", description="评估结果本地目录")
    progress_file: str = Field(default="./data/results/task_progress.json", description="进度存储文件")


class BaseEvaluationRaw(BaseModel):
    data_id: int = Field(description="数据唯一ID")
    raw_data: Dict = Field(default_factory=dict, description="原始数据字典")
    inputs: Dict = Field(default_factory=dict, description="服务输入数据")
    outputs: Dict = Field(default_factory=dict, description="服务输出数据")


class EvaluationData(BaseModel):
    data_list: List[BaseEvaluationRaw] = Field(default_factory=list)


class EvaluationResult(BaseModel):
    data_id: int = Field(description="数据唯一ID")
    metrics: Dict[str, float] = Field(default_factory=dict, description="单个结果的指标")


class EvaluationResults(BaseModel):
    results: List[EvaluationResult] = Field(default_factory=list)
    metrics: Dict = Field(default_factory=dict, description="性能指标")


class EvaluationContext(BaseModel):
    """评估上下文"""
    task_id: str = Field(description="任务ID")
    input_params: EvaluationServiceInputParams = Field(description="任务输入参数")
    temp_files: List[str] = Field(default_factory=list, description="临时文件列表")
    local_detail_path: Optional[str] = Field(default=None, description="本地详细结果路径")
    local_h5_detail_path: Optional[str] = Field(default=None, description="本地详细结果路径")

    def add_temp_file(self, file_path: str) -> None:
        """添加临时文件路径"""
        if file_path not in self.temp_files:
            self.temp_files.append(file_path)

    def set_local_detail_path(self, path: str) -> None:
        """设置本地详细结果路径"""
        self.local_detail_path = path
        self.add_temp_file(path)

    def set_local_h5_detail_path(self, path: str) -> None:
        """设置本地HDF5详细结果路径"""
        self.local_h5_detail_path = path
        self.add_temp_file(path)

    class Config:
        # 允许任意类型，因为我们有自定义方法
        arbitrary_types_allowed = True
        # 使用枚举值而不是名称
        use_enum_values = True



"""Utility modules for S3/MinIO integration."""

from .minio_client import MinioClient, MinioConfig
from .s3_handler import S3DataHandler

__all__ = [
    "MinioClient",
    "MinioConfig",
    "S3DataHandler",
]


"""Chart and visualization data types."""

from .dtypes import (
    BaseChart, RadarChart, BarChart, LineChart, TableChart,
    GaugeChart, ScatterChart, PieChart, HeatmapChart,
    AreaChart, WaterfallChart, BoxplotChart, HistogramChart,
    SankeyChart, TimelineChart, KPICardChart, ChartFactory
)

__all__ = [
    "BaseChart",
    "RadarChart",
    "BarChart",
    "LineChart",
    "TableChart",
    "GaugeChart",
    "ScatterChart",
    "PieChart",
    "HeatmapChart",
    "AreaChart",
    "WaterfallChart",
    "BoxplotChart",
    "HistogramChart",
    "SankeyChart",
    "TimelineChart",
    "KPICardChart",
    "ChartFactory",
]


"""Core modules for evaluation service base framework."""

from .base_service import BaseEvaluationService
from .exceptions import TaskCancelledException
from .common import (
    BaseEvaluationServiceConfig,
    EvaluationServiceInputParams,
    EvaluationContext,
    EvaluationData,
    EvaluationResult,
    EvaluationResults,
    BaseEvaluationRaw,
)
from .enum import TaskStatus, EvaluationStep
from .constants import ProgressConstants
from .models import TaskProgressState
from .managers import StepProgressManager
from .reporters import StepProgressReporter

__all__ = [
    "BaseEvaluationService",
    "TaskCancelledException",
    "BaseEvaluationServiceConfig",
    "EvaluationServiceInputParams",
    "EvaluationContext",
    "EvaluationData",
    "EvaluationResult",
    "EvaluationResults",
    "BaseEvaluationRaw",
    "TaskStatus",
    "EvaluationStep",
    "ProgressConstants",
    "TaskProgressState",
    "StepProgressManager",
    "StepProgressReporter",
]



import os
import threading
from typing import Dict, Any, Optional, List
from libentry import logger

from evaluation_service_base.core.enum import TaskStatus, EvaluationStep
from evaluation_service_base.core.models import TaskProgressState


class StepProgressManager:
    """步骤进度管理器"""

    def __init__(self, config):
        self.config = config
        self.task_states: Dict[str, TaskProgressState] = {}
        self.lock = threading.RLock()
        self._ensure_directory()

    def _ensure_directory(self):
        os.makedirs(os.path.dirname(self.config.progress_file), exist_ok=True)

    def create_task(self, task_id: str) -> bool:
        """创建任务"""
        with self.lock:
            if task_id in self.task_states:
                current_state = self.task_states[task_id]
                # 🆕 使用枚举比较
                if current_state.status == TaskStatus.RUNNING:
                    return False

            self.task_states[task_id] = TaskProgressState(task_id=task_id)
            logger.info(f"创建任务: {task_id}, 状态: {TaskStatus.PENDING.display_name}")
            return True

    def update_step_progress(self, task_id: str, step: EvaluationStep, progress: float, message: str = ""):
        """更新步骤进度"""
        with self.lock:
            if task_id not in self.task_states:
                logger.warning(f"任务 {task_id} 不存在，无法更新进度")
                return
            self.task_states[task_id].update_step_progress(step, progress, message)

    def complete_step(self, task_id: str, step: EvaluationStep, message: str = ""):
        """完成步骤"""
        with self.lock:
            if task_id not in self.task_states:
                logger.warning(f"任务 {task_id} 不存在，无法完成步骤")
                return
            self.task_states[task_id].complete_step(step, message)

    def complete_task(self, task_id: str, result: Optional[Dict] = None):
        """完成任务"""
        with self.lock:
            if task_id not in self.task_states:
                logger.warning(f"任务 {task_id} 不存在，无法标记完成")
                return

            state = self.task_states[task_id]
            state.mark_as_completed(result)
            logger.info(f"任务 {task_id} 已完成")

    def fail_task(self, task_id: str, error: str, error_info: Optional[Dict] = None):
        """任务失败"""
        with self.lock:
            if task_id not in self.task_states:
                logger.warning(f"任务 {task_id} 不存在，无法标记失败")
                return

            state = self.task_states[task_id]
            state.mark_as_failed(error, error_info)
            logger.error(f"任务 {task_id} 失败: {error}")

    def cancel_task(self, task_id: str) -> bool:
        """取消任务"""
        with self.lock:
            if task_id not in self.task_states:
                logger.warning(f"任务 {task_id} 不存在，无法取消")
                return False

            state = self.task_states[task_id]
            if not state.can_be_cancelled():
                logger.info(f"任务 {task_id} 状态为 {state.status.display_name}，无法取消")
                return False

            state.mark_as_cancelled()
            logger.info(f"任务 {task_id} 已被取消")
            return True

    def get_task_state(self, task_id: str) -> Optional[TaskProgressState]:
        """获取任务状态"""
        with self.lock:
            return self.task_states.get(task_id)

    def get_active_tasks(self) -> List[TaskProgressState]:
        """获取所有活跃任务"""
        with self.lock:
            return [state for state in self.task_states.values() if state.is_active()]

    def cleanup_finished_tasks(self, max_finished_tasks: int = 100) -> int:
        """清理已完成的任务，保留最近的一定数量"""
        with self.lock:
            finished_tasks = [
                (task_id, state) for task_id, state in self.task_states.items()
                if state.is_finished()
            ]

            if len(finished_tasks) <= max_finished_tasks:
                return 0

            # 按更新时间排序，保留最新的
            finished_tasks.sort(key=lambda x: x[1].updated_at, reverse=True)
            tasks_to_remove = finished_tasks[max_finished_tasks:]

            removed_count = 0
            for task_id, _ in tasks_to_remove:
                del self.task_states[task_id]
                removed_count += 1

            logger.info(f"清理了 {removed_count} 个已完成的任务")
            return removed_count




from typing import Optional, BinaryIO
import json
from minio import Minio, S3Error
from minio.commonconfig import CopySource
from dataclasses import dataclass


@dataclass
class MinioConfig:
    """Minio配置类"""
    endpoint: str
    access_key: str
    secret_key: str
    secure: bool = False
    cert_check: bool = False
    default_bucket: str = "default"
    forwarding_address: Optional[str] = None


class MinioClient:
    def __init__(self, config: MinioConfig):
        """
        初始化MinioClient
        Args:
            config: Minio配置对象
        """
        self.config = config
        try:
            self.client = Minio(
                endpoint=config.endpoint,
                access_key=config.access_key,
                secret_key=config.secret_key,
                secure=config.secure,
                cert_check=config.cert_check,
            )
        except Exception as e:
            print(f"s3 object storage connect error: {e}")
            raise e

    def is_bucket_exists(self, bucket_name: Optional[str] = None, auto_create: bool = True):
        """验证桶是否存在"""
        if bucket_name is None:
            bucket_name = self.config.default_bucket

        try:
            if not self.client.bucket_exists(bucket_name):
                print(f"s3 object storage bucket {bucket_name} does not exist")
                if auto_create:
                    self.client.make_bucket(bucket_name)
                    print(f"s3 object storage bucket {bucket_name} created")
        except S3Error as e:
            print(f"s3 object storage check bucket error: {e}")
            return False, "未查找到指定桶对象"
        return True, None

    def fput_object(
            self,
            object_name: str,
            file_path: str,
            bucket_name: Optional[str] = None,
            content_type: str = "application/json"
    ):
        """上传本地文件到S3对象存储"""
        if bucket_name is None:
            bucket_name = self.config.default_bucket

        try:
            self.client.fput_object(bucket_name, object_name, file_path, content_type)
            print(f"s3 object storage bucket {bucket_name} uploaded object {object_name}")
        except S3Error as e:
            print(f"s3 object storage upload object [{object_name}] error: {e}")
            return False, "对象文件上传失败"
        return True, None

    def fget_object(
            self,
            object_name: str,
            file_path: str,
            bucket_name: Optional[str] = None,
    ):
        """
        从S3对象存储下载文件到本地
        Args:
            object_name: 对象名称
            file_path: 本地文件保存路径
            bucket_name: 桶名称

        Returns:
            tuple: (成功标志, 错误信息)
        """
        if bucket_name is None:
            bucket_name = self.config.default_bucket

        try:
            # 确保本地目录存在
            import os
            os.makedirs(os.path.dirname(file_path), exist_ok=True)

            self.client.fget_object(bucket_name, object_name, file_path)
            print(f"s3 object storage bucket {bucket_name} downloaded object {object_name} to {file_path}")
        except S3Error as e:
            print(f"s3 object storage download object [{object_name}] error: {e}")
            return False, "对象文件下载失败"
        return True, None

    def put_object(
            self,
            object_name: str,
            object_content: BinaryIO,
            bucket_name: Optional[str] = None,
            object_length: int = -1,
            part_size: int = 5 * 1024 * 1024
    ):
        """
        上传文件流到s3对象存储
        Args:
            object_name: 对象名称
            object_content: 数据流对象
            bucket_name: 桶名称
            object_length: 数据流大小 (未知大小时使用-1)
            part_size: 分片大小 (数据流大小为-1时, 分片大小不能小于5MB)

        Returns:

        """
        if bucket_name is None:
            bucket_name = self.config.default_bucket
        try:
            self.client.put_object(bucket_name, object_name, object_content, length=object_length, part_size=part_size)
            print(f"s3 object storage bucket {bucket_name} uploaded object {object_name}")
        except S3Error as e:
            print(f"s3 object storage upload object [{object_name}] error: {e}")
            return False, "对象数据流上传失败"
        return True, None

    def download_object(
            self,
            object_name: str,
            bucket_name: Optional[str] = None,
    ):
        """
        从s3对象存储获取对象数据流
        Args:
            bucket_name: 桶名称
            object_name: 对象名称

        Returns:

        """
        if bucket_name is None:
            bucket_name = self.config.default_bucket
        try:
            s3_object = self.client.get_object(bucket_name, object_name)
        except S3Error as e:
            print(f"s3 object storage get object [{object_name}] error: {e}")
            return False, "对象数据流下载失败"
        return True, s3_object

    def list_object(
            self,
            prefix: Optional[str],
            bucket_name: Optional[str] = None,
            recursive=True
    ):
        """
        匹配获取对象列表
        Args:
            bucket_name: 桶名称
            prefix: 对象前缀
            recursive: 是否递归查询

        Returns:

        """
        if bucket_name is None:
            bucket_name = self.config.default_bucket
        object_list = []
        try:
            s3_object = self.client.list_objects(bucket_name, prefix=prefix, recursive=recursive)
            for _file in s3_object:
                object_list.append(_file)
        except S3Error as e:
            print(f"s3 object storage get object [{prefix}] error: {e}")
            return False, "对象列表获取失败"
        return True, object_list

    def state_object(
            self,
            object_name: str,
            bucket_name: Optional[str] = None,
    ):
        """
        获取对象存储状态信息
        Args:
            bucket_name: 桶名称
            object_name: 对象名称

        Returns:

        """
        if bucket_name is None:
            bucket_name = self.config.default_bucket
        try:
            s3_object = self.client.stat_object(bucket_name, object_name)
        except S3Error as e:
            print(f"s3 object storage get object [{object_name}] error: {e}")
            return False, "获取对象存储信息失败"
        return True, s3_object


    def delete_object(
            self,
            object_name: str,
            bucket_name: Optional[str] = None
    ):
        """
        从s3对象存储删除对象
        Args:
            bucket_name: 桶名称
            object_name: 对象名称

        Returns:
            bool: 删除是否成功
        """
        if bucket_name is None:
            bucket_name = self.config.default_bucket
        try:
            self.client.remove_object(bucket_name, object_name)
            print(f"s3 object storage deleted object {object_name}")
            return True
        except S3Error as e:
            print(f"s3 object storage delete object [{object_name}] error: {e}")
            return False


    def object_exists(
            self,
            object_name: str,
            bucket_name: Optional[str] = None
    ):
        """
        检查对象是否存在
        Args:
            object_name: 对象名称
            bucket_name: 桶名称

        Returns:
            bool: 对象是否存在
        """
        if bucket_name is None:
            bucket_name = self.config.default_bucket
        try:
            self.client.stat_object(bucket_name, object_name)
            return True
        except S3Error:
            return False

    def get_object_size(
            self,
            object_name: str,
            bucket_name: Optional[str] = None
    ):
        """
        获取对象大小
        Args:
            object_name: 对象名称
            bucket_name: 桶名称

        Returns:
            tuple: (成功标志, 大小或错误信息)
        """
        if bucket_name is None:
            bucket_name = self.config.default_bucket
        try:
            stat_info = self.client.stat_object(bucket_name, object_name)
            return True, stat_info.size
        except S3Error as e:
            print(f"s3 object storage get size [{object_name}] error: {e}")
            return False, "获取对象大小失败"

    def __del__(self):
        pass

if __name__ == '__main__':
    client = MinioClient()
    # 上传
    # print(client.fput_object(
    #     "tool_evaluation/infer_Result/Dataset7.n_RAG_e2e.json",
    #     file_path='data/Dataset7.n_RAG_e2e.json',
    #     content_type="application/json"
    # ))
    # 下载

    status, response = client.download_object(
        "tool_evaluation/infer_Result/Dataset7.n_RAG_e2e.json"
    )
    # 1. 读取内容
    content = response.read()  # 这是bytes

    # 2. 解码为字符串
    content_str = content.decode("utf-8")

    # 3. 解析为 JSON 对象
    data = json.loads(content_str)

    print(data)



"""
Evaluation Service Base Framework

A comprehensive framework for building evaluation services with:
- Progress tracking and task management
- Flexible evaluation data handling
- S3/MinIO storage integration
- Rich chart and reporting capabilities
- Extensible base service architecture

"""

__version__ = "0.1.0"
__author__ = "Your Name"
__email__ = "your.email@example.com"

# Core modules
from .core.base_service import BaseEvaluationService
from .core.exceptions import TaskCancelledException
from .core.common import (
    BaseEvaluationServiceConfig,
    EvaluationServiceInputParams,
    EvaluationContext,
    EvaluationData,
    EvaluationResult,
    EvaluationResults,
    BaseEvaluationRaw,
)
from .core.enum import TaskStatus, EvaluationStep
from .core.constants import ProgressConstants
from .core.models import TaskProgressState
from .core.managers import StepProgressManager
from .core.reporters import StepProgressReporter

# Utils modules
from .utils.minio_client import MinioClient, MinioConfig
from .utils.s3_handler import S3DataHandler

# Charts modules
from .charts.dtypes import (
    BaseChart, RadarChart, BarChart, LineChart, TableChart,
    GaugeChart, ScatterChart, PieChart, HeatmapChart,
    AreaChart, WaterfallChart, BoxplotChart, HistogramChart,
    SankeyChart, TimelineChart, KPICardChart, ChartFactory
)

__all__ = [
    # Core
    "BaseEvaluationService",
    "TaskCancelledException",
    "BaseEvaluationServiceConfig",
    "EvaluationServiceInputParams",
    "EvaluationContext",
    "EvaluationData",
    "EvaluationResult",
    "EvaluationResults",
    "BaseEvaluationRaw",
    "TaskStatus",
    "EvaluationStep",
    "ProgressConstants",
    "TaskProgressState",
    "StepProgressManager",
    "StepProgressReporter",

    # Utils
    "MinioClient",
    "MinioConfig",
    "S3DataHandler",

    # Charts
    "BaseChart",
    "RadarChart",
    "BarChart",
    "LineChart",
    "TableChart",
    "GaugeChart",
    "ScatterChart",
    "PieChart",
    "HeatmapChart",
    "AreaChart",
    "WaterfallChart",
    "BoxplotChart",
    "HistogramChart",
    "SankeyChart",
    "TimelineChart",
    "KPICardChart",
    "ChartFactory",
]



from datetime import datetime
from typing import Dict, Any, Optional
from dataclasses import dataclass, field

from evaluation_service_base.core.enum import TaskStatus, EvaluationStep


@dataclass
class TaskProgressState:
    """任务进度状态"""
    task_id: str
    status: TaskStatus = TaskStatus.PENDING  # 🆕 使用枚举
    overall_progress: int = 0
    current_step: Optional[EvaluationStep] = None
    current_step_progress: float = 0.0
    message: str = ""
    created_at: datetime = field(default_factory=datetime.now)
    updated_at: datetime = field(default_factory=datetime.now)
    error_info: Optional[Dict[str, Any]] = None
    result: Optional[Dict[str, Any]] = None
    cancelled: bool = False

    def to_dict(self) -> Dict[str, Any]:
        """转换为字典"""
        return {
            "task_id": self.task_id,
            "status": self.status.value,  # ✅ 正确使用
            "status_display": self.status.display_name,  # ✅ 正确使用
            "overall_progress": self.overall_progress,
            "current_step": {
                "name": self.current_step.display_name if self.current_step else None,
                "progress": self.current_step_progress
            } if self.current_step else None,
            "message": self.message,
            "created_at": self.created_at.isoformat(),
            "updated_at": self.updated_at.isoformat(),
            "cancelled": self.cancelled
        }

    def update_step_progress(self, step: EvaluationStep, step_progress: float, message: str = ""):
        """更新当前步骤进度"""
        self.current_step = step
        self.current_step_progress = max(0, min(100, step_progress))
        self.overall_progress = step.calculate_overall_progress(self.current_step_progress)

        if message:
            self.message = f"{step.display_name}: {message}"
        else:
            self.message = f"{step.display_name}: {self.current_step_progress:.1f}%"

        self.updated_at = datetime.now()

        # 🆕 使用枚举比较
        if self.status == TaskStatus.PENDING and step_progress > 0:
            self.status = TaskStatus.RUNNING

    def complete_step(self, step: EvaluationStep, message: str = ""):
        """完成步骤"""
        self.update_step_progress(step, 100.0, message or "完成")

    def mark_as_completed(self, result: Optional[Dict] = None):
        """标记为完成"""
        self.status = TaskStatus.COMPLETED
        self.overall_progress = 100
        self.message = "任务完成"
        self.result = result
        self.updated_at = datetime.now()

    def mark_as_failed(self, error: str, error_info: Optional[Dict] = None):
        """标记为失败"""
        self.status = TaskStatus.FAILED
        self.message = error
        self.error_info = error_info
        self.updated_at = datetime.now()

    def mark_as_cancelled(self):
        """标记为已取消"""
        self.status = TaskStatus.CANCELLED
        self.cancelled = True
        self.message = "任务已被取消"
        self.updated_at = datetime.now()

    def can_be_cancelled(self) -> bool:
        """判断是否可以被取消"""
        return self.status == TaskStatus.RUNNING

    def is_active(self) -> bool:
        """判断任务是否处于活跃状态"""
        return self.status in [TaskStatus.PENDING, TaskStatus.RUNNING]

    def is_finished(self) -> bool:
        """判断任务是否已结束"""
        return self.status in [TaskStatus.COMPLETED, TaskStatus.FAILED, TaskStatus.CANCELLED]




import json
import io
import traceback
import pandas as pd
import h5py
from typing import Dict, Any, Generator, Optional, Union, List
import tempfile
import os

from liblogging import logger



class S3DataHandler:
    def __init__(self, minio_client):
        self.minio_client = minio_client

    # ======================== JSON 操作 ========================
    def _read_json_from_s3(self, s3_path: str) -> Dict[str, Any]:
        """从S3读取JSON文件 - 通用实现"""
        status, response = self.minio_client.download_object(s3_path)
        if not status:
            raise Exception(f"读取S3文件失败: {s3_path}")

        content = response.read()
        try:
            content_str = content.decode("utf-8")
            data = json.loads(content_str)
            return data
        except UnicodeDecodeError:
            raise Exception(f"文件 {s3_path} 不是有效的UTF-8文本文件")
        except json.JSONDecodeError as e:
            raise Exception(f"文件 {s3_path} 不是有效的JSON格式: {e}")

    def _put_json_to_s3(self, json_object, s3_path: str):
        """写入JSON到S3 - 通用实现"""
        try:
            json_stream = io.BytesIO(json.dumps(json_object, ensure_ascii=False, indent=2).encode('utf-8'))
            self.minio_client.put_object(s3_path, json_stream)
        except Exception as e:
            raise Exception(f"写入S3文件失败: {s3_path}, 错误: {str(e)}")

    # ======================== HDF5 本地操作 ========================
    def read_hdf_chunks(self, file_path: str, key: str = 'df', chunk_size: int = 100) -> Generator[
        pd.DataFrame, None, None]:
        """
        分片读取 HDF5 文件，以生成器方式返回每片数据。

        Args:
            file_path: HDF5 文件路径
            key: HDF5 中存储 DataFrame 的键
            chunk_size: 每次读取的行数

        Returns:
            生成器，每次返回一个 DataFrame 片段
        """
        if chunk_size <= 0:
            raise ValueError("chunk_size 必须大于 0")

        if not os.path.exists(file_path):
            raise FileNotFoundError(f"HDF5文件不存在: {file_path}")

        with pd.HDFStore(file_path, mode='r') as store:
            # 检查是否存在该 key
            if key not in store:
                available_keys = list(store.keys())
                raise KeyError(f"HDF5 文件中未找到键: {key}，可用键: {available_keys}")

            storer = store.get_storer(key)
            # 获取总量
            total_rows = storer.nrows
            print(f"总行数: {total_rows}, 每批次: {chunk_size}")

            start = 0
            batch_num = 0
            while start < total_rows:
                end = min(start + chunk_size, total_rows)
                batch_num += 1
                print(f"读取批次 {batch_num}: 行 {start}-{end - 1}")
                yield store.select(key, start=start, stop=end)
                start = end

    def append_to_hdf5(self, df: pd.DataFrame, file_path: str, key: str = 'df'):
        """将DataFrame数据存入HDF5文件中
        Args:
            df: 待追加的DataFrame数据
            file_path: HDF5文件路径
            key: HDF5文件存储键
        """
        if df.empty:
            print("警告: DataFrame为空，跳过写入")
            return

        def safe_json_dumps(x):
            # 首先检查None
            if x is None:
                return None

            # 安全检查是否为NaN
            try:
                if pd.isna(x):
                    return None
            except (TypeError, ValueError):
                # 如果pd.isna()失败，继续处理
                pass

            # 如果已经是基本类型，直接返回
            if isinstance(x, (str, int, float, bool)):
                return x

            # 尝试JSON序列化
            try:
                return json.dumps(x, ensure_ascii=False)
            except (TypeError, ValueError):
                return str(x)

        df = df.copy()  # 避免修改原始数据

        # 处理object类型列
        object_cols = df.select_dtypes(include='object').columns
        if len(object_cols) > 0:
            print(f"处理object类型列: {list(object_cols)}")
            for col in object_cols:
                df[col] = df[col].apply(safe_json_dumps)

        # 创建目录（如果不存在）
        os.makedirs(os.path.dirname(file_path), exist_ok=True)

        try:
            with pd.HDFStore(file_path, mode='a') as store:
                if key in store:
                    # 已存在数据
                    store.append(key, df, format='table', data_columns=True)
                else:
                    store.put(key, df, format='table', data_columns=True)

        except ValueError as e:
            if "column has a limit of" in str(e):
                logger.warning(f"检测到列宽不足，将执行动态扩展操作... 错误: {e}")
                # 读取旧数据
                df_old = pd.DataFrame()
                with pd.HDFStore(file_path, mode='a') as store_read:
                    df_old = store_read.get(key)
                # 合并新旧数据
                df_combined = pd.concat([df_old, df], ignore_index=True)
                # 创建副本
                backup_path = file_path + '.bak'
                os.rename(file_path, backup_path)
                # 尝试写入新数据
                try:
                    with pd.HDFStore(file_path, mode='w') as store_write:
                        store_write.put(key, df_combined, format='table', data_columns=True)

                    logger.info("动态扩展成功，数据已重写。")
                    # 成功后删除备份
                    os.remove(backup_path)
                except Exception as write_e:
                    # 如果重写失败，恢复备份
                    logger.error(f"重写扩容HDF5文件失败: {write_e}。正在从备份恢复...")
                    os.rename(backup_path, file_path)
                    raise write_e
            else:
                raise e
        except Exception as e:
            logger.error(f"将数据追加到HDF5文件失败: {str(e)}\n{traceback.format_exc()}")

    # ======================== HDF5 S3 操作 ========================
    def _read_hdf5_from_s3(self, s3_path: str) -> Dict[str, Any]:
        """从S3读取HDF5文件并返回所有数据"""
        status, response = self.minio_client.download_object(s3_path)
        if not status:
            raise Exception(f"读取S3文件失败: {s3_path}")

        content = response.read()

        # 验证是否为HDF5文件
        if not content.startswith(b'\x89HDF'):
            raise Exception(f"文件 {s3_path} 不是HDF5格式")

        try:
            with h5py.File(io.BytesIO(content), 'r') as h5_file:
                return self._extract_all_hdf5_data(h5_file)
        except Exception as e:
            raise Exception(f"读取HDF5文件失败: {e}")

    def _extract_all_hdf5_data(self, h5_file) -> Dict[str, Any]:
        """提取HDF5文件的所有数据"""
        result = {
            'datasets': {},
            'groups': {},
            'attributes': dict(h5_file.attrs),
            'structure': []
        }

        def extract_item(name, obj):
            result['structure'].append(name)
            if isinstance(obj, h5py.Dataset):
                # 对于大数据集，只返回元信息
                if obj.size > 100000:  # 超过10万个元素
                    data_info = f"Large dataset: shape {obj.shape}, size {obj.size}"
                else:
                    data_info = obj[...].tolist()

                result['datasets'][name] = {
                    'data': data_info,
                    'shape': obj.shape,
                    'dtype': str(obj.dtype),
                    'size': obj.size,
                    'attributes': dict(obj.attrs)
                }
            elif isinstance(obj, h5py.Group):
                result['groups'][name] = {
                    'attributes': dict(obj.attrs),
                    'keys': list(obj.keys())
                }

        h5_file.visititems(extract_item)
        return result

    def read_hdf5_chunks_from_s3(self, s3_path: str, key: str = 'df', chunk_size: int = 100) -> Generator[
        pd.DataFrame, None, None]:
        """从S3下载HDF5文件并分片读取"""
        # 创建临时文件
        with tempfile.NamedTemporaryFile(suffix='.h5', delete=False) as temp_file:
            temp_path = temp_file.name

        try:
            # 下载文件到临时位置
            print(f"从S3下载HDF5文件: {s3_path}")
            status = self.minio_client.fget_object(s3_path, temp_path)
            if not status:
                raise Exception(f"从S3下载文件失败: {s3_path}")

            # 分片读取
            yield from self.read_hdf_chunks(temp_path, key, chunk_size)

        finally:
            # 清理临时文件
            if os.path.exists(temp_path):
                os.unlink(temp_path)
                print(f"已清理临时文件: {temp_path}")

    def _put_hdf5_to_s3(self, file_path: str, s3_path: str):
        """上传HDF5文件到S3"""
        if not os.path.exists(file_path):
            raise FileNotFoundError(f"本地文件不存在: {file_path}")

        try:
            self.minio_client.fput_object(
                s3_path,
                file_path=file_path,
                content_type="application/x-hdf5"
            )
            print(f"HDF5文件上传成功: {file_path} -> {s3_path}")
        except Exception as e:
            raise Exception(f"上传HDF5文件到S3失败: {s3_path}, 错误: {str(e)}")

    # ======================== 通用文件操作 ========================
    def _put_detail_to_s3(self, file_path: str, s3_path: str, content_type: str = "application/json"):
        """写入文件到S3 - 通用实现"""
        self._put_file_to_s3(file_path, s3_path, content_type)

    def _put_file_to_s3(self, file_path: str, s3_path: str, content_type: str = "application/json"):
        """写入文件到S3 - 通用实现"""
        if not os.path.exists(file_path):
            raise FileNotFoundError(f"本地文件不存在: {file_path}")

        try:
            self.minio_client.fput_object(
                s3_path,
                file_path=file_path,
                content_type=content_type
            )
            print(f"文件上传成功: {file_path} -> {s3_path}")
        except Exception as e:
            raise Exception(f"写入S3文件失败: {s3_path}, 错误: {str(e)}")

    def _download_file_from_s3(self, s3_path: str, local_path: str):
        """从S3下载文件到本地"""
        try:
            # 创建本地目录（如果不存在）
            os.makedirs(os.path.dirname(local_path), exist_ok=True)

            status = self.minio_client.fget_object(s3_path, local_path)
            if not status:
                raise Exception(f"下载失败")
            print(f"文件下载成功: {s3_path} -> {local_path}")
        except Exception as e:
            raise Exception(f"从S3下载文件失败: {s3_path}, 错误: {str(e)}")

    # ======================== 实用方法 ========================
    def get_file_info_from_s3(self, s3_path: str) -> Dict[str, Any]:
        """获取S3文件信息"""
        try:
            status, response = self.minio_client.download_object(s3_path)
            if not status:
                raise Exception(f"文件不存在: {s3_path}")

            content = response.read()
            file_info = {
                'size': len(content),
                'type': 'unknown'
            }

            # 检测文件类型
            if content.startswith(b'\x89HDF'):
                file_info['type'] = 'hdf5'
            elif content.strip().startswith(b'{') or content.strip().startswith(b'['):
                file_info['type'] = 'json'
            elif b',' in content and b'\n' in content:
                file_info['type'] = 'csv'

            return file_info
        except Exception as e:
            raise Exception(f"获取文件信息失败: {s3_path}, 错误: {str(e)}")

    def list_hdf5_keys(self, file_path: str) -> List[str]:
        """列出HDF5文件中的所有键"""
        if not os.path.exists(file_path):
            raise FileNotFoundError(f"HDF5文件不存在: {file_path}")

        with pd.HDFStore(file_path, mode='r') as store:
            return list(store.keys())

    def get_hdf5_info(self, file_path: str, key: str = None) -> Dict[str, Any]:
        """获取HDF5文件信息"""
        if not os.path.exists(file_path):
            raise FileNotFoundError(f"HDF5文件不存在: {file_path}")

        info = {'keys': [], 'details': {}}

        with pd.HDFStore(file_path, mode='r') as store:
            info['keys'] = list(store.keys())

            if key:
                if key in store:
                    storer = store.get_storer(key)
                    info['details'][key] = {
                        'nrows': storer.nrows,
                        'ncols': len(storer.attrs.values_axes[0]),
                        'columns': list(storer.attrs.values_axes[0])
                    }
                else:
                    raise KeyError(f"键 '{key}' 不存在于HDF5文件中")
            else:
                # 获取所有键的信息
                for k in info['keys']:
                    storer = store.get_storer(k)
                    info['details'][k] = {
                        'nrows': storer.nrows,
                        'ncols': len(storer.attrs.values_axes[0]),
                        'columns': list(storer.attrs.values_axes[0])
                    }

        return info



from typing import List, Dict, Any, Optional, Union
from pydantic import BaseModel, Field

# ==================== 图表基础模型 ====================

class BaseChart(BaseModel):
    """图表基础模型"""
    chart_type: str = Field(description="图表类型标识")
    title: str = Field(description="图表标题")
    subtitle: Optional[str] = Field(default=None, description="副标题")
    description: Optional[str] = Field(default=None, description="图表说明/解读")
    id: Optional[str] = Field(default=None, description="图表唯一标识（可用于前端定位/锚点）")


class RadarChart(BaseChart):
    """雷达图模型"""
    chart_type: str = Field(default="radar", description="图表类型：雷达图")
    metrics: List[str] = Field(description="指标名称列表")
    legend: List[str] = Field(description="图例")
    data: List[Dict[str, Any]] = Field(description="数据，格式：[{name: '系列名', value: [数值列表]}]")
    max_values: List[float] = Field(default=None, description="每个指标的最大值，默认为100")

    # 前端渲染参数
    grid_lines: bool = Field(default=True, description="是否显示网格线")
    fill_area: bool = Field(default=True, description="是否填充区域")

    def __init__(self, **data):
        if data.get('max_values') is None:
            data['max_values'] = [100.0] * len(data.get('metrics', []))
        super().__init__(**data)


class BarChart(BaseChart):
    chart_type: str = Field(default="bar", description="图表类型：分组柱状图")
    categories: List[str] = Field(description="X轴类别名称（主分组）")
    legend: List[str] = Field(description="图例（子分组）")
    data: List[Dict[str, Any]] = Field(description="数据，格式：[{name: '图例名', values: [各类别数值]}]")
    y_axis_label: Optional[str] = Field(None, description="Y轴标签")
    x_axis_label: Optional[str] = Field(None, description="X轴标签")

    orientation: str = Field(default="vertical", description="方向：vertical(垂直) 或 horizontal(水平)")
    stacked: bool = Field(default=False, description="是否堆叠显示")
    show_values: bool = Field(default=True, description="是否显示数值标签")


class LineChart(BaseChart):
    """线图模型"""
    chart_type: str = Field(default="line", description="图表类型：线图")
    x_axis: List[str] = Field(description="X轴数据")
    legend: List[str] = Field(description="图例")
    data: List[Dict[str, Any]] = Field(description="数据，格式：[{name: '系列名', values: [数值列表]}]")
    y_axis_label: Optional[str] = Field(None, description="Y轴标签")
    x_axis_label: Optional[str] = Field(None, description="X轴标签")

    # 前端渲染参数
    smooth: bool = Field(default=True, description="是否平滑曲线")
    show_points: bool = Field(default=True, description="是否显示数据点")
    fill_area: bool = Field(default=False, description="是否填充区域")
    area_opacity: float = Field(default=0.25, description="面积填充透明度，0-1")


class TableChart(BaseChart):
    """表格模型"""
    chart_type: str = Field(default="table", description="图表类型：表格")
    columns:  List[Dict]= Field(description="列定义，格式：[{key: '字段名', label: '显示名', width?: '宽度'}]")
    data: List[Dict[str, Any]] = Field(description="表格数据")



class GaugeChart(BaseChart):
    """仪表盘模型"""
    chart_type: str = Field(default="gauge", description="图表类型：仪表盘")
    metrics: List[Dict[str, Any]] = Field(
        description="指标数据，格式：[{name: '指标名', value: 数值, max: 最大值, unit?: '单位'}]")

    # 前端渲染参数
    show_pointer: bool = Field(default=True, description="是否显示指针")
    color_ranges: Optional[List[Dict[str, Any]]] = Field(default=None, description="颜色范围设置")
    start_angle: Optional[int] = Field(default=None, description="起始角度")
    end_angle: Optional[int] = Field(default=None, description="结束角度")


class ScatterChart(BaseChart):
    """散点图模型"""
    chart_type: str = Field(default="scatter", description="图表类型：散点图")
    x_axis_label: str = Field(description="X轴标签")
    y_axis_label: str = Field(description="Y轴标签")
    legend: List[str] = Field(description="图例")
    data: List[Dict[str, Any]] = Field(
        description="数据，格式：[{name: '系列名', points: [{x: x值, y: y值, size?: 大小}]}]")

    # 前端渲染参数
    show_regression_line: bool = Field(default=False, description="是否显示回归线")
    point_size: int = Field(default=8, description="数据点大小")
    x_log: bool = Field(default=False, description="X轴对数刻度")
    y_log: bool = Field(default=False, description="Y轴对数刻度")


class PieChart(BaseChart):
    """饼图模型"""
    chart_type: str = Field(default="pie", description="图表类型：饼图")
    data: List[Dict[str, Any]] = Field(description="数据，格式：[{name: '名称', value: 数值, color?: '颜色'}]")

    # 前端渲染参数
    show_legend: bool = Field(default=True, description="是否显示图例")
    show_labels: bool = Field(default=True, description="是否显示标签")
    show_percentages: bool = Field(default=True, description="是否显示百分比")
    donut_mode: bool = Field(default=False, description="是否为环形图")
    inner_radius: Optional[float] = Field(default=None, description="环形图内半径比例 0-1")
    rose_type: Optional[str] = Field(default=None, description="南丁格尔图类型: area/radius")


class HeatmapChart(BaseChart):
    """热力图模型"""
    chart_type: str = Field(default="heatmap", description="图表类型：热力图")
    x_axis: List[str] = Field(description="X轴标签")
    y_axis: List[str] = Field(description="Y轴标签")
    data: List[List[float]] = Field(description="数据矩阵")

    # 前端渲染参数
    color_scheme: str = Field(default="viridis", description="颜色方案")
    show_values: bool = Field(default=True, description="是否显示数值")
    value_format: Optional[str] = Field(default=None, description="数值格式，如 '0.0%' or '.2f'")


# ==================== 新增图表类型 ====================

class AreaChart(BaseChart):
    """堆叠面积图/面积图"""
    chart_type: str = Field(default="area", description="图表类型：面积图")
    x_axis: List[str] = Field(description="X轴数据")
    legend: List[str] = Field(description="图例")
    data: List[Dict[str, Any]] = Field(description="数据，格式：[{name: '系列名', values: [数值列表]}]")
    y_axis_label: Optional[str] = Field(None, description="Y轴标签")
    x_axis_label: Optional[str] = Field(None, description="X轴标签")
    stacked: bool = Field(default=True, description="是否堆叠")
    area_opacity: float = Field(default=0.25, description="面积透明度")


class WaterfallChart(BaseChart):
    """瀑布图，用于展示增减变化"""
    chart_type: str = Field(default="waterfall", description="图表类型：瀑布图")
    categories: List[str] = Field(description="类别/阶段名称")
    values: List[float] = Field(description="对应数值，正负表示增减")
    total_label: Optional[str] = Field(default="总计", description="总计标签")


class BoxplotChart(BaseChart):
    """箱线图，用于展示分布"""
    chart_type: str = Field(default="boxplot", description="图表类型：箱线图")
    categories: List[str] = Field(description="类别名称")
    data: List[Dict[str, Any]] = Field(description="数据: [{name: 类别, values: [min, q1, median, q3, max], outliers?: [[x,y], ...]}]")
    y_axis_label: Optional[str] = Field(default=None, description="Y轴标签")


class HistogramChart(BaseChart):
    """直方图"""
    chart_type: str = Field(default="histogram", description="图表类型：直方图")
    bins: List[str] = Field(description="区间标签，如 '0-10', '10-20'")
    counts: List[int] = Field(description="每个区间的数量")
    density: bool = Field(default=False, description="是否显示概率密度")


class SankeyChart(BaseChart):
    """桑基图，展示流向关系"""
    chart_type: str = Field(default="sankey", description="图表类型：桑基图")
    nodes: List[Dict[str, Any]] = Field(description="节点列表，如 [{name: 'A'}, {name: 'B'}]")
    links: List[Dict[str, Any]] = Field(description="边列表，如 [{source: 0, target: 1, value: 10}]")


class TimelineChart(BaseChart):
    """时间轴/甘特图"""
    chart_type: str = Field(default="timeline", description="图表类型：时间轴/甘特图")
    items: List[Dict[str, Any]] = Field(description="条目: [{label, start, end, group?, color?}]")
    groups: Optional[List[str]] = Field(default=None, description="分组，可选")


class KPICardChart(BaseChart):
    """KPI卡片，用于展示核心指标数值"""
    chart_type: str = Field(default="kpi_card", description="图表类型：KPI卡片")
    items: List[Dict[str, Any]] = Field(description="KPI列表: [{label, value, unit?, delta?, trend?}]")


# ==================== 图表工厂类 ====================

class ChartFactory:
    """图表工厂类，用于创建不同类型的图表"""

    CHART_TYPES = {
        "radar": RadarChart,
        "bar": BarChart,
        "line": LineChart,
        "table": TableChart,
        "gauge": GaugeChart,
        "scatter": ScatterChart,
        "pie": PieChart,
        "heatmap": HeatmapChart,
        "area": AreaChart,
        "waterfall": WaterfallChart,
        "boxplot": BoxplotChart,
        "histogram": HistogramChart,
        "sankey": SankeyChart,
        "timeline": TimelineChart,
        "kpi_card": KPICardChart
    }

    @classmethod
    def create_chart(cls, chart_type: str, **kwargs) -> BaseChart:
        """根据类型创建图表"""
        if chart_type not in cls.CHART_TYPES:
            raise ValueError(f"不支持的图表类型: {chart_type}")

        chart_class = cls.CHART_TYPES[chart_type]
        return chart_class(**kwargs)

    @classmethod
    def get_supported_types(cls) -> List[str]:
        """获取支持的图表类型"""
        return list(cls.CHART_TYPES.keys())
